
# ------------------------------------------------- #
#               General formulas
# ------------------------------------------------- #
colMax <- function(data = NULL) {
    result <- apply(data,2,max)
    return(result)
} 

colMin <- function(data = NULL) {
  result <- apply(data,2,min)
  return(result)
}

################
# stdfromstd() #
################
# Function that calculates the new standard deviation when means
# with their own standard deviations are summed. To sum standard
# deviations one uses the following formula:
# Square root(std1^2 + std2^2 + ... + stdn^2)
stdfromstd <- function(stdevs = NULL){
  vars <- stdevs^2
  sum <- sum(vars, na.rm = TRUE)
  std <- sqrt(sum)
  return(std)
}

###########################
# Function: findoverlap() #
###########################
# This function finds the fraction of values in matrix1
# which are higher than the values in matrix2 when randomized.
# If this fraction is >90 or <10 it means >90% of the values of matrix 1
# are different from matrix 2 in the same direction (either lower or higher)
# and therefore the difference between the two sample sets is significant.
findoverlap <- function(matrix1, matrix2, seed1, seed2){
  # Randomize order of rows in both matrices, unless stated otherwise (seed1 = 0)
  if(seed1 != 0){
    set.seed(seed1)
    rand1 <- sample(nrow(matrix1))
    matrix1 <- matrix1[rand1,,drop = FALSE]
    set.seed(seed2)
    rand2 <- sample(nrow(matrix2))
    matrix2 <- matrix2[rand2,,drop = FALSE]
  }
  
  # Find fraction of rows where value in M1 > M2.
  fractions <- rep(NA, length(ncol(matrix1)))
  
  for(i in 1:ncol(matrix1)){
    fractions[i] <- length(which(matrix1[,i] > matrix2[,i]))/length(matrix1[,i])
  }
  
  return(fractions)
}

# ------------------------------------------------- #
#               Get matrices
# ------------------------------------------------- #
#####################
# findParallelFlows #
#####################
# Input flows = readLIM$flows and a flow number.
# Returns vector with row numbers in readLIM$flows of all flows
# with the same source and sink as the flow defined by flownr
# (so all parallel flows).
findParallelFlows <- function(flows = NULL, flownr = NULL) {
  sc1 <- which(flows[,1] == flows[flownr, 1]) # flows with same source compartment
  sc2 <- which(flows[,2] == flows[flownr, 2]) # flows with same sink compartment
  sc3 <- c(sc1, sc2) # all flows ...
  same <- sc3[duplicated(sc3)] # if a flownr occurs twice it has the same source and sink
  return(same)
}

#################
# getFlowMatrix #
#################
# This function creates a flowmatrix with flow values.
# It produces the same as Flowmatrix() - which is a build in function -, but
# takes into the account the possibility that multiple flows may occur between compartments.
# For every flow it checks whether there are other flows with the same source and sink
# compartments. The values of all flows with the same source and sink are summed to
# obtain the total flow between those two compartments. The total flow value is stored
# at the location of the original flownumber. So if flow1 and flow2 both occur between
# compartment A and B, the total flow value at index 1 is flow1 + flow2 and the total flow value
# at index 2 is also flow1 + flow2.
# It then replaces all flownumbers in the original flowmatrix with the corresponding total flow
# value. So, if flow1 is not included in the flowmatrix, but flow2 is, then the total flux value at
# index 2 is taken, which already also includes the flow value of flow1.

getFlowMatrix <- function (originalFM = NULL, flows = NULL, answers = NULL) {
  # Check function input
  if(is.null(originalFM)) {stop("Supply flowmatrix with flownumbers from LIM$Flowmatrix")}
  if(is.null(flows)) {stop("Supply the flows dataframe from readLIM$flows")}
  if(is.null(answers)) {stop("Supply a vector with all flow values")}
  
  # Create a vector which will store all total flow values
  totalflows <- numeric(length(flows[,1]))
  # Create a vector with all flownumbers
  flownrs <- c(1:length(flows[,1]))
  
  for(flownr in flownrs) {
    same <- findParallelFlows(flows = flows, flownr = flownr)
    totalflow <- sum(answers[same], na.rm = TRUE) # sum the flow values of all flows with the same source and sink.
    totalflows[flownr] <- totalflow # store the value at the index of the original flow
  }
  
  # Find the locations in the matrix with flownumbers
  indices <- match(flownrs, originalFM)
  indices <- indices[!is.na(indices)] # remove NA's (no matches) from the list.
  # Replace the flownumbers in the original matrix with the corresponding
  # total flow values (so if the flownr is 3 in the matrix, but it should be flow
  # 2 and 3, it will insert the totalflow of flow 2 and 3).
  FM <- replace(originalFM, indices, totalflows[originalFM[indices]])
  # Return a flowmatrix with flow values.
  return(FM)
}


# ------------------------------------------------- #
#               Plotting functions
# ------------------------------------------------- #

############
# getNodes #
############
getNodes <- function(site = NULL, LIM, TrLevels){

  #########
  # Nodes #
  #########
  # ----- Create data frame -----
  nodeID <- c(1:LIM$NComponents)
  nodes <- data.frame(nodeID)
  
  # add ID
  nodes <- cbind(nodes, LIM$Components)
  nodes$plotname <- nodeID
  
  # add biomass
  colnames(nodes)[3] <- "biomass"
  biomNA <- which(is.na(nodes$biomass))
  biomZero <- which(nodes$biomass == 0)
  nodes$biomass[biomNA] <- 0 # Biomasses of NA set to zero for plotting purposes
  
  # add trophic levels
  nodes$TL <- colMeans(TrLevels)
  # artificially change TL of detritus for plotting purposes
  nodes[which(nodes$name == "DOCPHYTO_S"), "TL"] <- 1.5
  nodes[which(nodes$name == "PHYTO_W"), "TL"] <- 1.5
  
  nodes[which(nodes$name == "PHYTO_S"), "TL"] <- 1.20
  nodes[which(nodes$name == "SLAB_S"), "TL"] <- 1.15
  nodes[which(nodes$name == "REFRAC"), "TL"] <- 1.10
  
  # add size classes and classID
  classes <- c(rep("Detritus/DOC",5),
               "Prokaryote",
               rep("Detritus/DOC",3),
               rep("Meiofauna", 8),
               rep("Macrofauna", 9),
               rep("Megafauna", 12),
               rep("Fish", 3))
  classID <- c(rep("1",5), # Det
               "2", # Prokaryote
               rep("1",3), # Det
               rep("3", 8), # Meio
               rep("4", 9), # Macro
               rep("5", 12), # Mega
               rep("6", 3)) # Fish
  nodes$class <- classes
  nodes$classID <- classID
  
  # Add feeding strategies
  nodes$feeding <- 0
  nodes$feeding[1:9] <- "NA"
  nodes$feeding[c(10,11,14,15,19,20,24,27:31,33,35)] <- "Deposit feeder"
  nodes$feeding[c(12,16,17,21,22,25,26,36:41)] <- "Carnivore"
  nodes$feeding[c(13,18,23,32,34)] <- "Filter feeder"
  
  # Take log of Biomass to compress plot.
  nodes$logBM <- log10(nodes$biomass)
  nodes$logBM[biomNA] <- c(-1.5, -1.5, -3.5, -3.5, -2.5) # Biomasses of NA set artificially for plotting purposes
  
  # Set values for non-existent compartments to zero.
  nodes$logBM[biomZero] <- 0 # If no biomass, logBM is also zero.
  nodes$TL[biomZero] <- 0
  
  return(nodes)
}

############
# getEdges #
############
getEdges <- function(site = NULL, readLIM, Flows){
  
  # ------- Create Data frame --------
  links <- readLIM$flows[,1:3]
  links$weight <- Flows$mean
  
  # Sum all flows between two compartments
  iters <- length(which(duplicated(links[,1:2]) == TRUE))
  for(i in 1:iters) {
    duplicates <- which(duplicated(links[,1:2]) == TRUE)
    if(length(duplicates) > 0) {
      same <- which(links$from == links$from[duplicates[1]] & 
                      links$to == links$to[duplicates[1]])
      links[same[1],4] <- sum(links$weight[same])
      same <- same[-1]
      links <- links[-same,]
    }
    else {break}
  }
  
  # Remove all external flows
  external <- which(links$from < 0 | links$to < 0)
  links <- links[-external,]
  
  # Remove all flows with weight zero
  noflow <- which(links$weight == 0)
  if(length(noflow) > 0){links <- links[-noflow,]}
  
  rownames(links) <- NULL
  
  return(links)
}

###################
# plotResolvedWeb #
###################
# This function plots the resolved foodweb as a function of biomass
# against trophic level of each compartment.

plotResolvedWeb <- function(site = NULL,
                            LIM, readLIM, Flows, TrLevels){
  require(igraph)
  require(RColorBrewer)
  
  nodes <- getNodes(site = site, LIM = LIM, TrLevels = TrLevels)
  links <- getEdges(site = site, readLIM = readLIM, Flows = Flows)
  
  ####################
  # plot with iGraph #
  ####################
  # http://www.kateto.net/polnet2017
  # parameters
  font_size <- 1 #1.5
  colour_scheme <- "Set3" # view options: display.brewer.all()
  circle_size <- 27 #22
  
  # ---- Create matrix ("net") with igraph package ----
  net <- graph_from_data_frame(d=links, vertices=nodes, directed=T) 
  
  # ----Colour (size class) and shape (detritus) nodes ---
  # number of categories = nr of colours needed
  nrclrs <- length(unique(V(net)$class)) 
  colrsV <- brewer.pal(nrclrs, colour_scheme)
  # color nodes according to size class
  V(net)$color <- colrsV[as.numeric(V(net)$classID)] 
  V(net)$frame.color <- "white" # colour of the line around the vertices
  V(net)$shape <- "circle"
  V(net)[c(2, 4, 7, 8, 9)]$shape <- "square"

  # ---- Other node (vertex) properties ----
  V(net)$label <- V(net)$plotname # no node labels / compartment names
  V(net)$label.font <- 1 # font for label inside circle 1 = plain, 2 = bold, 3, italic, 4 bold italic, 5 symbol
  V(net)$label.family <- "Helvetica"
  V(net)$label.cex <- font_size  # size of label inside circle
  V(net)$label.color <- "black" # colour of label inside circle
  V(net)$size <- circle_size # 
  
  # Delete labels that are not easily visible (added manually as text)
  if(site == "ref"){
    V(net)$label[c(6, 10, 12, 13, 18, 21, 27, 28, 31)] <- NA
  }else if(site == "undist"){
    V(net)$label[c(6, 12, 16, 17, 19, 27)] <- NA
  }else if(site == "dist"){
    V(net)$label[c(27, 30)] <- NA
  }
  
  # ---- Flow (edge) properties ----
  arrow_col <- "grey40"
  E(net)$color <- arrow_col
  E(net)$arrow.size <- 0.5 # smaller arrow size
  E(net)$width <- sqrt(sqrt(E(net)$weight))*10
  
  # Position nodes on x=BM, y=TL
  l <- matrix(c(nodes$logBM, nodes$TL),ncol=2)
  
  par(cex.lab = font_size, cex.axis = font_size)
  plot(net,
       layout=l, rescale = FALSE, axes = TRUE,
       xlab = expression(paste("log"["10"],"C stock (mmol C m"^"-2",")")), 
       ylab = "Trophic Level",
       asp = 0, xlim = c(-4,4), ylim = c(0.8,4)
  )
  legend(title="Compartment type",
         x=2.5, y=4, 
         c(unique(V(net)$class)), col="#777777",
         pch=21, pt.bg=colrsV, pt.cex=3, cex=font_size, bty="n", ncol=1)
  legend(title = "Flow size",
         x=2.5, y=2.9,
         c(paste(sprintf("%.2e", 0.000001)), 
           paste(sprintf("%.2f", 1.00))), 
         col = c(arrow_col, arrow_col),
         lwd = c(sqrt(sqrt(0.000001))*10,
                 sqrt(sqrt(1.00))*10), 
         pt.cex=2, cex = font_size, bty="n")
  # Add labels that aren't visible otherwise +
  # Add names of detritus pools
  if(site == "ref"){
    # labels
    text(c(0.75, -0.6, -0.93,-1.25, -0.86, -0.6, -1.3, -1.1, -1.16), 
         c(1.97, 2.1,  3.1 , 1.9,  1.98, 3.25,   2.22,  2.1,  2.22), 
         c("6", "10", "12",  "13", "18", "21",  "27", "28",  "31"),
         cex = font_size, font = 1) # 2 is bold
    # detritus
    text(c(-3.5, -3.5,  -3.5, -3.5,  -1.5, -1.5,  -1.5, -1.5,  0.7, 0.7,  2.2, 2.2,  3.4, -3), 
         c(0.89, 0.78,  1.39, 1.28,  0.89, 0.78,  1.39, 1.28,  0.89, 0.78,  0.89, 0.78,  0.89, 2.6), 
         c("DOC", "(sDet+rDet)", "DOC", "(lDet)", "Suspended", "semi-labile det.", "Suspended", "labile det.", 
           "Sedimentary", "labile det.", "Sedimentary", "semi-labile det.", "Refractory det.", "Carcass"),
         cex = font_size)
    # ABC
    text(-4, 3.8, "(a)", cex = 3)
  }else if(site == "undist"){
    # labels
    text(c(1.03, -0.6, -0.43,-1.25, -0.22, -1.12), 
         c(2.0,  3.05, 2.24,  3.3,  2.33, 2.12), 
         c("6", "12", "16", "17", "19", "27"),
         cex = font_size)
    # detritus
    text(c(-3.5, -3.5,  -3.5, -3.5,  -1.5, -1.5,  -1.5, -1.5,  0.7, 0.7,  2.2, 2.2,  3.4, -3), 
         c(0.89, 0.78,  1.39, 1.28,  0.89, 0.78,  1.39, 1.28,  0.89, 0.78,  0.89, 0.78,  0.89, 2.55), 
         c("DOC", "(sDet+rDet)", "DOC", "(lDet)", "Suspended", "semi-labile det.", "Suspended", "labile det.", 
           "Sedimentary", "labile det.", "Sedimentary", "semi-labile det.", "Refractory det.", "Carcass"),
         cex = font_size)
    # ABC
    text(-4, 3.8, "(b)", cex = 3)
  }else if(site == "dist"){
    # labels
    text(c(-1.15, -1.65), 
         c(2.12, 2.12), 
         c("27", "30"),
         cex = font_size)
    # detritus
    text(c(-3.5, -3.5,  -3.5, -3.5,  -1.5, -1.5,  -1.5, -1.5,  0.7, 0.7,  2.2, 2.2,  3.4, -3), 
         c(0.89, 0.78,  1.39, 1.28,  0.89, 0.78,  1.39, 1.28,  0.89, 0.78,  0.89, 0.78,  0.89, 2.65), 
         c("DOC", "(sDet+rDet)", "DOC", "(lDet)", "Suspended", "semi-labile det.", "Suspended", "labile det.", 
           "Sedimentary", "labile det.", "Sedimentary", "semi-labile det.", "Refractory det.", "Carcass"),
         cex = font_size)
    # ABC
    text(-4, 3.8, "(c)", cex = 3)
  }
}

# ------------------------------------------------- #
#               Trophic Levels
# ------------------------------------------------- #

# getTrophInd
# is a copy of the TrophInd function from the NetInd package. In the original 
# function the TL is the average of the trophic levels of the input compartments
# plus 1. We want the carrion compartment to have a trophic level equal to the average
# trophic levels of all input compartments, but without the addition of 1, because
# the carrion pool does not really 'feed' on anything. We also don't just want to
# set it to trophic level 1.
getTrophInd <- function (Flow = NULL, Tij = t(Flow), Import = NULL, Export = NULL, 
                         Dead = NULL) 
{
  if (is.character(Dead)) {
    dead <- which(rownames(Tij) %in% Dead)
  }else {dead <- Dead}
  if (length(dead) != length(Dead)) {
    stop("Dead not recognized")
  }
  N <- NetIndices:::InternalNetwork(Tij, Import, Export)
  p <- NetIndices:::Diet(N$Tint, dead, N$iN)
  ncomp <- ncol(N$Tint)
  A <- -p
  diag(A) <- 1
  B <- rep(1, ncomp)
  B[9] <- 0 # Carcass has index 9, should not add 1 but add 0.
  TL <- ginv(A) %*% B
  OI <- vector(length = ncomp)
  for (i in 1:ncomp) OI[i] <- sum((TL - (TL[i] - 1))^2 * p[i, 
                                                           ])
  return(data.frame(TL, OI, row.names = rownames(N$Tint)))
}

